@@ -42,15 +42,310 @@ class Exception_TestCase(scaffold.Exception_TestCase):
 
         self.valid_exceptions = {
             daemon.daemon.DaemonError: dict(
-                min_args = 0,
+                min_args = 1,
                 types = (Exception,),
             ),
+            daemon.daemon.DaemonEnvironmentOSError: dict(
+                min_args = 1,
+                types = (daemon.daemon.DaemonError, OSError),
+            ),
             daemon.daemon.DaemonProcessDetachError: dict(
-                min_args = 0,
+                min_args = 1,
                 types = (daemon.daemon.DaemonError, OSError),
             ),
         }
 
+
+class change_working_directory_TestCase(scaffold.TestCase):
+    """ Test cases for change_working_directory function """
+
+    def setUp(self):
+        """ Set up test fixtures """
+        self.mock_outfile = StringIO()
+        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+
+        scaffold.mock(
+            "os.chdir",
+            tracker=self.mock_tracker)
+
+        self.test_directory = object()
+        self.test_args = dict(
+            directory=self.test_directory,
+            )
+
+    def tearDown(self):
+        """ Tear down test fixtures """
+        scaffold.mock_restore()
+
+    def test_changes_working_directory_to_specified_directory(self):
+        """ Should change working directory to specified directory. """
+        args = self.test_args
+        directory = self.test_directory
+        expect_mock_output = """\
+            Called os.chdir(%(directory)r)
+            """ % vars()
+        daemon.daemon.change_working_directory(**args)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
+    def test_raises_daemon_error_on_os_error(self):
+        """ Should raise a DaemonError on receiving and OSError. """
+        args = self.test_args
+        test_error = OSError(errno.ENOENT, "No such directory")
+        os.chdir.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        self.failUnlessRaises(
+            expect_error,
+            daemon.daemon.change_working_directory, **args)
+
+    def test_error_message_contains_original_error_message(self):
+        """ Should raise a DaemonError with original message. """
+        args = self.test_args
+        test_error = OSError(errno.ENOENT, "No such directory")
+        os.chdir.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        try:
+            daemon.daemon.change_working_directory(**args)
+        except expect_error, exc:
+            pass
+        self.failUnlessIn(str(exc), str(test_error))
+
+
+class change_root_directory_TestCase(scaffold.TestCase):
+    """ Test cases for change_root_directory function """
+
+    def setUp(self):
+        """ Set up test fixtures """
+        self.mock_outfile = StringIO()
+        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+
+        scaffold.mock(
+            "os.chdir",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "os.chroot",
+            tracker=self.mock_tracker)
+
+        self.test_directory = object()
+        self.test_args = dict(
+            directory=self.test_directory,
+            )
+
+    def tearDown(self):
+        """ Tear down test fixtures """
+        scaffold.mock_restore()
+
+    def test_changes_working_directory_to_specified_directory(self):
+        """ Should change working directory to specified directory. """
+        args = self.test_args
+        directory = self.test_directory
+        expect_mock_output = """\
+            Called os.chdir(%(directory)r)
+            ...
+            """ % vars()
+        daemon.daemon.change_root_directory(**args)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
+    def test_changes_root_directory_to_specified_directory(self):
+        """ Should change root directory to specified directory. """
+        args = self.test_args
+        directory = self.test_directory
+        expect_mock_output = """\
+            ...
+            Called os.chroot(%(directory)r)
+            """ % vars()
+        daemon.daemon.change_root_directory(**args)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
+    def test_raises_daemon_error_on_os_error_from_chdir(self):
+        """ Should raise a DaemonError on receiving an OSError from chdir. """
+        args = self.test_args
+        test_error = OSError(errno.ENOENT, "No such directory")
+        os.chdir.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        self.failUnlessRaises(
+            expect_error,
+            daemon.daemon.change_root_directory, **args)
+
+    def test_raises_daemon_error_on_os_error_from_chroot(self):
+        """ Should raise a DaemonError on receiving an OSError from chroot. """
+        args = self.test_args
+        test_error = OSError(errno.EPERM, "No chroot for you!")
+        os.chroot.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        self.failUnlessRaises(
+            expect_error,
+            daemon.daemon.change_root_directory, **args)
+
+    def test_error_message_contains_original_error_message(self):
+        """ Should raise a DaemonError with original message. """
+        args = self.test_args
+        test_error = OSError(errno.ENOENT, "No such directory")
+        os.chdir.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        try:
+            daemon.daemon.change_root_directory(**args)
+        except expect_error, exc:
+            pass
+        self.failUnlessIn(str(exc), str(test_error))
+
+
+class change_file_creation_mask_TestCase(scaffold.TestCase):
+    """ Test cases for change_file_creation_mask function """
+
+    def setUp(self):
+        """ Set up test fixtures """
+        self.mock_outfile = StringIO()
+        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+
+        scaffold.mock(
+            "os.umask",
+            tracker=self.mock_tracker)
+
+        self.test_mask = object()
+        self.test_args = dict(
+            mask=self.test_mask,
+            )
+
+    def tearDown(self):
+        """ Tear down test fixtures """
+        scaffold.mock_restore()
+
+    def test_changes_umask_to_specified_mask(self):
+        """ Should change working directory to specified directory. """
+        args = self.test_args
+        mask = self.test_mask
+        expect_mock_output = """\
+            Called os.umask(%(mask)r)
+            """ % vars()
+        daemon.daemon.change_file_creation_mask(**args)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
+    def test_raises_daemon_error_on_os_error_from_chdir(self):
+        """ Should raise a DaemonError on receiving an OSError from umask. """
+        args = self.test_args
+        test_error = OSError(errno.EINVAL, "Whatchoo talkin' 'bout?")
+        os.umask.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        self.failUnlessRaises(
+            expect_error,
+            daemon.daemon.change_file_creation_mask, **args)
+
+    def test_error_message_contains_original_error_message(self):
+        """ Should raise a DaemonError with original message. """
+        args = self.test_args
+        test_error = OSError(errno.ENOENT, "No such directory")
+        os.umask.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        try:
+            daemon.daemon.change_file_creation_mask(**args)
+        except expect_error, exc:
+            pass
+        self.failUnlessIn(str(exc), str(test_error))
+
+
+class change_process_owner_TestCase(scaffold.TestCase):
+    """ Test cases for change_process_owner function """
+
+    def setUp(self):
+        """ Set up test fixtures """
+        self.mock_outfile = StringIO()
+        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+
+        scaffold.mock(
+            "os.setuid",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "os.setgid",
+            tracker=self.mock_tracker)
+
+        self.test_uid = object()
+        self.test_gid = object()
+        self.test_args = dict(
+            uid=self.test_uid,
+            gid=self.test_gid,
+            )
+
+    def tearDown(self):
+        """ Tear down test fixtures """
+        scaffold.mock_restore()
+
+    def test_changes_gid_and_uid_in_order(self):
+        """ Should change process GID and UID in correct order.
+
+            Since the process requires appropriate privilege to use
+            either of `setuid` or `setgid`, changing the UID must be
+            done last.
+
+            """
+        args = self.test_args
+        expect_mock_output = """\
+            Called os.setgid(...)
+            Called os.setuid(...)
+            """ % vars()
+        daemon.daemon.change_process_owner(**args)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
+    def test_changes_group_id_to_gid(self):
+        """ Should change process GID to specified value. """
+        args = self.test_args
+        gid = self.test_gid
+        expect_mock_output = """\
+            Called os.setgid(%(gid)r)
+            ...
+            """ % vars()
+        daemon.daemon.change_process_owner(**args)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
+    def test_changes_user_id_to_uid(self):
+        """ Should change process UID to specified value. """
+        args = self.test_args
+        uid = self.test_uid
+        expect_mock_output = """\
+            ...
+            Called os.setuid(%(uid)r)
+            """ % vars()
+        daemon.daemon.change_process_owner(**args)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
+    def test_raises_daemon_error_on_os_error_from_setgid(self):
+        """ Should raise a DaemonError on receiving an OSError from setgid. """
+        args = self.test_args
+        test_error = OSError(errno.EPERM, "No switching for you!")
+        os.setgid.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        self.failUnlessRaises(
+            expect_error,
+            daemon.daemon.change_process_owner, **args)
+
+    def test_raises_daemon_error_on_os_error_from_setuid(self):
+        """ Should raise a DaemonError on receiving an OSError from setuid. """
+        args = self.test_args
+        test_error = OSError(errno.EPERM, "No switching for you!")
+        os.setuid.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        self.failUnlessRaises(
+            expect_error,
+            daemon.daemon.change_process_owner, **args)
+
+    def test_error_message_contains_original_error_message(self):
+        """ Should raise a DaemonError with original message. """
+        args = self.test_args
+        test_error = OSError(errno.EINVAL, "Whatchoo talkin' 'bout?")
+        os.setuid.mock_raises = test_error
+        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        try:
+            daemon.daemon.change_process_owner(**args)
+        except expect_error, exc:
+            pass
+        self.failUnlessIn(str(exc), str(test_error))
+
 
 class prevent_core_dump_TestCase(scaffold.TestCase):
     """ Test cases for prevent_core_dump function """
@@ -651,15 +946,31 @@ class redirect_stream_TestCase(scaffold.TestCase):
         """ Set up test fixtures """
         setup_streams_fixtures(self)
 
+        self.test_system_stream = FakeFileDescriptorStringIO()
+        self.test_target_stream = FakeFileDescriptorStringIO()
+        self.test_null_file = FakeFileDescriptorStringIO()
+
+        def mock_open(path, flag, mode=None):
+            if path == os.devnull:
+                result = self.test_null_file.fileno()
+            else:
+                raise OSError(errno.NOENT, "No such file", path)
+            return result
+
+        scaffold.mock(
+            "os.open",
+            returns_func=mock_open,
+            tracker=self.mock_tracker)
+
     def tearDown(self):
         """ Tear down test fixtures """
         scaffold.mock_restore()
 
-    def test_duplicates_file_descriptor(self):
-        """ Should duplicate file descriptor from target to system stream """
-        system_stream = FakeFileDescriptorStringIO()
+    def test_duplicates_target_file_descriptor(self):
+        """ Should duplicate file descriptor from target to system stream. """
+        system_stream = self.test_system_stream
         system_fileno = system_stream.fileno()
-        target_stream = FakeFileDescriptorStringIO()
+        target_stream = self.test_target_stream
         target_fileno = target_stream.fileno()
         expect_mock_output = """\
             Called os.dup2(%(target_fileno)r, %(system_fileno)r)
@@ -668,6 +979,23 @@ class redirect_stream_TestCase(scaffold.TestCase):
         self.failUnlessOutputCheckerMatch(
             expect_mock_output, self.mock_outfile.getvalue())
 
+    def test_duplicates_null_file_descriptor_by_default(self):
+        """ Should by default duplicate the null file to the system stream. """
+        system_stream = self.test_system_stream
+        system_fileno = system_stream.fileno()
+        target_stream = None
+        null_path = os.devnull
+        null_flag = os.O_RDWR
+        null_file = self.test_null_file
+        null_fileno = null_file.fileno()
+        expect_mock_output = """\
+            Called os.open(%(null_path)r, %(null_flag)r)
+            Called os.dup2(%(null_fileno)r, %(system_fileno)r)
+            """ % vars()
+        daemon.daemon.redirect_stream(system_stream, target_stream)
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
 
 class make_default_signal_map_TestCase(scaffold.TestCase):
     """ Test cases for make_default_signal_map function """
@@ -774,7 +1102,6 @@ def setup_daemon_context_fixtures(testcase):
     setup_pidfile_fixtures(testcase)
 
     testcase.mock_pidfile_path = tempfile.mktemp()
-
     testcase.mock_pidlockfile = scaffold.Mock(
         "pidlockfile.PIDLockFile",
         tracker=testcase.mock_tracker)
@@ -784,47 +1111,10 @@ def setup_daemon_context_fixtures(testcase):
         "daemon.daemon.is_detach_process_context_required",
         returns=True,
         tracker=testcase.mock_tracker)
-    scaffold.mock(
-        "daemon.daemon.detach_process_context",
-        tracker=testcase.mock_tracker)
-    scaffold.mock(
-        "daemon.daemon.prevent_core_dump",
-        tracker=testcase.mock_tracker)
-    scaffold.mock(
-        "daemon.daemon.close_all_open_files",
-        tracker=testcase.mock_tracker)
-    scaffold.mock(
-        "daemon.daemon.redirect_stream",
-        tracker=testcase.mock_tracker)
     scaffold.mock(
         "daemon.daemon.make_default_signal_map",
         returns=object(),
         tracker=testcase.mock_tracker)
-    scaffold.mock(
-        "daemon.daemon.set_signal_handlers",
-        tracker=testcase.mock_tracker)
-
-    testcase.mock_stderr = FakeFileDescriptorStringIO()
-
-    testcase.daemon_context_args = dict(
-        stdin = testcase.stream_files_by_name['stdin'],
-        stdout = testcase.stream_files_by_name['stdout'],
-        stderr = testcase.stream_files_by_name['stderr'],
-        )
-    testcase.test_instance = daemon.DaemonContext(
-        **testcase.daemon_context_args)
-
-    def mock_open(filename, mode=None, buffering=None):
-        if filename in testcase.stream_files_by_path:
-            result = testcase.stream_files_by_path[filename]
-        else:
-            result = FakeFileDescriptorStringIO()
-        return result
-
-    scaffold.mock(
-        "__builtin__.open",
-        returns_func=mock_open,
-        tracker=testcase.mock_tracker)
 
     scaffold.mock(
         "os.getuid",
@@ -835,16 +1125,13 @@ def setup_daemon_context_fixtures(testcase):
         returns=object(),
         tracker=testcase.mock_tracker)
 
-    scaffold.mock(
-        "sys.stdin",
-        tracker=testcase.mock_tracker)
-    scaffold.mock(
-        "sys.stdout",
-        tracker=testcase.mock_tracker)
-    scaffold.mock(
-        "sys.stderr",
-        mock_obj=testcase.mock_stderr,
-        tracker=testcase.mock_tracker)
+    testcase.daemon_context_args = dict(
+        stdin = testcase.stream_files_by_name['stdin'],
+        stdout = testcase.stream_files_by_name['stdout'],
+        stderr = testcase.stream_files_by_name['stderr'],
+        )
+    testcase.test_instance = daemon.DaemonContext(
+        **testcase.daemon_context_args)
 
 
 class DaemonContext_TestCase(scaffold.TestCase):
@@ -1027,7 +1314,34 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         """ Set up test fixtures """
         setup_daemon_context_fixtures(self)
 
-        self.test_files_preserve = object()
+        scaffold.mock(
+            "daemon.daemon.detach_process_context",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.change_working_directory",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.change_root_directory",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.change_file_creation_mask",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.change_process_owner",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.prevent_core_dump",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.close_all_open_files",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.redirect_stream",
+            tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.set_signal_handlers",
+            tracker=self.mock_tracker)
+
         self.test_files_preserve_fds = object()
         scaffold.mock(
             "daemon.daemon.DaemonContext._get_exclude_file_descriptors",
@@ -1041,25 +1355,46 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             tracker=self.mock_tracker)
 
         scaffold.mock(
-            "os.umask",
+            "sys.stdin",
             tracker=self.mock_tracker)
         scaffold.mock(
-            "os.chdir",
+            "sys.stdout",
             tracker=self.mock_tracker)
         scaffold.mock(
-            "os.chroot",
-            tracker=self.mock_tracker)
-        scaffold.mock(
-            "os.setuid",
-            tracker=self.mock_tracker)
-        scaffold.mock(
-            "os.setgid",
+            "sys.stderr",
             tracker=self.mock_tracker)
 
     def tearDown(self):
         """ Tear down test fixtures """
         scaffold.mock_restore()
 
+    def test_performs_steps_in_expected_sequence(self):
+        """ Should perform daemonisation steps in expected sequence. """
+        instance = self.test_instance
+        instance.chroot_directory = object()
+        instance.detach_process = True
+        instance.pidfile = self.mock_pidlockfile
+        expect_mock_output = """\
+            Called daemon.daemon.change_root_directory(...)
+            Called daemon.daemon.prevent_core_dump()
+            Called daemon.daemon.change_file_creation_mask(...)
+            Called daemon.daemon.change_working_directory(...)
+            Called daemon.daemon.change_process_owner(...)
+            Called daemon.daemon.detach_process_context()
+            Called daemon.daemon.DaemonContext._make_signal_handler_map()
+            Called daemon.daemon.set_signal_handlers(...)
+            Called daemon.daemon.DaemonContext._get_exclude_file_descriptors()
+            Called daemon.daemon.close_all_open_files(...)
+            Called daemon.daemon.redirect_stream(...)
+            Called daemon.daemon.redirect_stream(...)
+            Called daemon.daemon.redirect_stream(...)
+            Called pidlockfile.PIDLockFile.__enter__()
+            """ % vars()
+        self.mock_outfile.truncate(0)
+        instance.open()
+        self.failUnlessOutputCheckerMatch(
+            expect_mock_output, self.mock_outfile.getvalue())
+
     def test_changes_root_directory_to_chroot_directory(self):
         """ Should change root directory to `chroot_directory` option. """
         instance = self.test_instance
@@ -1067,8 +1402,8 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         instance.chroot_directory = chroot_directory
         expect_mock_output = """\
             ...
-            Called os.chdir(%(chroot_directory)r)
-            Called os.chroot(%(chroot_directory)r)
+            Called daemon.daemon.change_root_directory(
+                %(chroot_directory)r)
             ...
             """ % vars()
         instance.open()
@@ -1084,7 +1419,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         scaffold.mock_restore()
         self.failIfIn(
             self.mock_outfile.getvalue(),
-            "Called os.chroot")
+            "Called daemon.daemon.change_root_directory")
 
     def test_prevents_core_dump(self):
         """ Should request prevention of core dumps """
@@ -1121,7 +1456,8 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         instance.working_directory = working_directory
         expect_mock_output = """\
             ...
-            Called os.chdir(%(working_directory)r)
+            Called daemon.daemon.change_working_directory(
+                %(working_directory)r)
             ...
             """ % vars()
         instance.open()
@@ -1136,7 +1472,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         instance.umask = umask
         expect_mock_output = """\
             ...
-            Called os.umask(%(umask)r)
+            Called daemon.daemon.change_file_creation_mask(%(umask)r)
             ...
             """ % vars()
         instance.open()
@@ -1144,29 +1480,16 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         self.failUnlessOutputCheckerMatch(
             expect_mock_output, self.mock_outfile.getvalue())
 
-    def test_changes_user_id_to_uid(self):
-        """ Should change process UID to `uid` option. """
+    def test_changes_owner_to_specified_uid_and_gid(self):
+        """ Should change process UID and GID to `uid` and `gid` options. """
         instance = self.test_instance
         uid = object()
-        instance.uid = uid
-        expect_mock_output = """\
-            ...
-            Called os.setuid(%(uid)r)
-            ...
-            """ % vars()
-        instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
-
-    def test_changes_group_id_to_gid(self):
-        """ Should change process GID to `gid` option. """
-        instance = self.test_instance
         gid = object()
+        instance.uid = uid
         instance.gid = gid
         expect_mock_output = """\
             ...
-            Called os.setgid(%(gid)r)
+            Called daemon.daemon.change_process_owner(%(uid)r, %(gid)r)
             ...
             """ % vars()
         instance.open()
